# frozen_string_literal: true

require 'rspec/core/example_group'

module RSpec
  module Core
    class ExampleGroup
      def described_method
        metadata = self.class.metadata
        return metadata[:described_method] if metadata[:described_method]

        klass = described_class
        return nil unless klass.is_a? Class

        recurse_metadata do |cur|
          method = cur[:description].strip.gsub(%r{[^0-9A-Za-z_!?=+\-/*<>]}, '')
          if klass.respond_to?(method) || klass.method_defined?(method)
            metadata[:described_method] = method.to_sym
            false
          else
            true
          end
        end

        metadata[:described_method]
      end

      private

      def recurse_metadata(&block)
        metadata = self.class.metadata
        metadata = metadata[:parent_example_group] while metadata && block.call(metadata)
      end
    end
  end
end
