# frozen_string_literal: true

require 'rspec/unit_ext/core_ext/rspec/core/example_group/metadata'
require_relative 'expectation/attributes'

module RSpec
  module UnitExt
    class Expectation
      include Attributes

      def initialize(example_group, **attributes)
        @example_group = example_group
        override_attributes attributes
      end

      def run
        evaluator = evaluator_proc
        matcher   = matcher_inst
        failure   = failure_proc

        if matcher.supports_block_expectations?
          @example_group.expect(&evaluator).to matcher, failure
        else
          @example_group.expect(evaluator.call).to matcher, failure
        end
      end

      private

      def evaluator_proc
        lambda do
          @result = if args.is_a? Array
                      base.send method, *args
                    elsif args
                      base.send method, args
                    else
                      base.send method
                    end
        end
      end

      def matcher_inst
        @example_group.send matcher, expect
      end

      def failure_proc
        base = self.base.inspect

        lambda do
          <<-FAILURE
            Unit Expectation Failed:
                  Call:   #{base}.#{method}(#{args})
              Expected:   #{expect}
                   Got:   #{@result}
          FAILURE
        end
      end
    end
  end
end
