# frozen_string_literal: true

module RSpec
  module UnitExt
    class Expectation
      module Attributes
        ATTRS          = %i[base method args matcher expect].freeze
        INFERRED_ATTRS = %i[base method matcher].freeze

        (ATTRS - INFERRED_ATTRS).each do |key|
          define_method key do
            @attributes[key]
          end
        end

        ATTRS.each do |key|
          define_method "#{key}=" do |val|
            @attributes[key] = val
          end
        end

        def base
          base = @attributes[:base] || @example_group.described_class

          if base.is_a? Proc
            base.call
          else
            base
          end
        end

        def method
          @attributes[:method] || @example_group.described_method
        end

        def matcher
          @attributes[:matcher] || :eq
        end

        def fill_attributes(**attributes)
          @attributes ||= {}
          @attributes   = attributes.slice(*ATTRS).merge @attributes
        end

        def override_attributes(**attributes)
          @attributes ||= {}
          @attributes   = @attributes.merge attributes.slice(*ATTRS)
        end
      end
    end
  end
end
